/* ReSCAL - GTK callbacks
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include <gtk/gtk.h>

#include "defs.h"
#include "macros_public.h"
#include "entry.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "rescal.h"
#include "monitor.h"
#ifdef PARALLEL
#include "synchro.h"
#endif
#include "view.h"
#include "terre.h"

#ifdef COLLISIONS
#include "collisions.h"
#endif

extern int H, L, P;
extern gint area_w, area_h;
extern float frame_delay;
extern unsigned long int iter, md_iter;
extern unsigned char opt_quit;
extern double temps;
#ifdef COLLISIONS
extern int col_iter;
extern float meanvel, maxvel;
#endif
#ifdef PARALLEL
extern int mode_par;    //mode parallele
extern int proc_id;     //numero de process en mode parallele (0 = serveur)
#endif

gint timer=0;
gint end_of_rescal=0;     //semaphore (pour l'arret des callbacks)
gint rescal_paused=0;     //semaphore (pour la mise en pause)
GdkGC *gc=NULL;
pthread_mutex_t mutex_pause;
pthread_cond_t cond_pause;

void callbacks_init()
{
  /* Initialize mutex and condition variable objects */
  pthread_mutex_init(&mutex_pause, NULL);
  pthread_cond_init (&cond_pause, NULL);
}

double get_flash_delay(gpointer  data)
{
  int i;
  double t0, t1;

  elapsed(&t0);
  for (i=1; i<10; i++) gcallback_flash(data);
  elapsed(&t1);
  return (t1-t0)/10.0;
}


void *rescal_thread(void *data) {
  sleep(1);
  rescal(0);
  end_of_rescal = 1;
  pthread_exit(0);
  return 0;
}

//#define CPT_MAX (1000/FLASH_DELAY) //10

gboolean gcallback_dump (gpointer data){
  DumpPar  *dp;
  dp = (DumpPar*)data;
  switch(dp->type){
    case(DUMP_BIN):
      dump_terre(dp->delay);
#ifdef DUMP_RUGOSI
      dump_rugosi();
#endif
#if defined(COLLISIONS) && !defined(ROTATIONS)
      dump_mvt(dp->delay);
      dump_grad_vel(dp->delay);
#endif
      break;
#ifdef DUMP_IMAGE
    case(DUMP_PNG):
      view_dump_inter(dp->delay, IMG_PNG);
      break;
    case(DUMP_JPEG):
      view_dump_inter(dp->delay, IMG_JPEG);
      break;
#endif
    case(DUMP_INFO):
      dump_info();
      break;
  }
  return !end_of_rescal;
}

void check_pause()
{
  if (rescal_paused){
    pthread_cond_wait(&cond_pause, &mutex_pause);
  }
}

gboolean gcallback_pause (gpointer  data){
  if (!rescal_paused){
    rescal_paused = 1;
    LogPrintf("pause\n");
    gtk_button_set_label((GtkButton*)data, "Continue");
  }
  else{
    rescal_paused = 0;
    LogPrintf("continue\n");
    pthread_cond_signal(&cond_pause);
    gtk_button_set_label((GtkButton*)data, "Pause");
  }
  return 1;
}

gboolean gcallback_stop (gpointer  data){
  LogPrintf("time to quit !\n");
  exit(-1);
}

gboolean gcallback_quit (gpointer data)
{
  if (opt_quit && end_of_rescal) {
    LogPrintf("quit\n");
    exit(-1);
  }

  return 1;
}

gboolean gcallback_flash (gpointer  data){
  static char start=2;
  static int last_iter=0;
  static int cpt=1;
  static float speed=0;
  static double t0, t1=0, t3=0;//, e0, e1, e2;
  static float avg_delay=1e10;
  static char slabel[256], slabel2[256];
#ifdef COLLISIONS
  static float speed_col=0;
  static int last_col_iter=0;
#endif

  //elapsed(&e0);
  GtkWidget  *drawingarea = (GtkWidget  *)data;
  if (!drawingarea) return FALSE;

  GdkDrawable *drawable;
  static GdkRgbCmap colormap;

  drawable = drawingarea -> window;
  if (!drawable) return FALSE;

  if (!gc) {
    //LogPrintf("!gc\n");
    gdk_rgb_init();
    gc = gdk_gc_new (drawable);
    view_palette((int*)colormap.colors);
  }
  unsigned char *image;
  image = view();
  gdk_draw_indexed_image (drawable, gc, 0, 0, area_w, area_h, GDK_RGB_DITHER_NONE, image, area_w, &colormap);

  /* Display text area */
  elapsed(&t0);
  if (!t1) t1 = t0;
  if (!t3) t3 = t0;

  if (t0-t1 >= 1.0) {
    float delay = t0-t1;
    avg_delay = delay/(float)cpt;
    speed = 60.0*(float)(iter-last_iter)/delay;
    last_iter = iter;
    t1 = t0;
    cpt = 1;
    if (start){
      start--;
      if (!start) LogPrintf("affichage : %.2f fps\n", (1.0/avg_delay));
    }
  }
  else {
    cpt++;
  }

#ifdef COLLISIONS
  if (t0-t3 >= 3.0) {
    float delay = t0-t3;
    speed_col = (float)60.0*(col_iter-last_col_iter)/delay;
    last_col_iter = col_iter;
    t3 = t0;
  }
#endif

  //sprintf(slabel,"Transitions = %lu%09lu \nTemps=%f \nVitesse = %d tr/min \nAffichage = %.2f img/sec", md_iter, iter, temps, (int)speed, (1.0/avg_delay));
  sprintf(slabel,"Size = %dx%dx%d \nTime = %.3f \nTransitions = %lu%09lu \nTrans. rate = %d tr/min \nFrame rate = %.2f fps", H, L, P, temps, md_iter, iter, (int)speed, (1.0/avg_delay));
#ifdef COLLISIONS
  sprintf(slabel2,"\nCollision rate = %d cyc/min \nMean vel. = %.3f \nMax. vel. = %.3f", (int)speed_col, meanvel, maxvel);
  strcat(slabel,slabel2);
#endif
#ifdef PARALLEL
  if (mode_par){
    sprintf(slabel2,"\nPid = %d", proc_id);
    strcat(slabel,slabel2);
  }
#endif
  gtk_label_set_text(GTK_LABEL(lookup_widget(GTK_WIDGET (drawingarea), "label2")), slabel);
  //gdk_gc_unref (gc);
  /*elapsed(&e1);
  LogPrintf("flash_delay = %f\telapsed = %f\tcpt = %d\n", (e1-e0)*1000.0, (e0-e2)*1000.0, cpt);
  e2 = e1;*/
  return TRUE;
}

gboolean on_drawingarea1_expose_event  (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data) {
  if(!timer) {
    timer_init(widget);
  }

  if (area_w) {
    gcallback_flash(lookup_widget(GTK_WIDGET (widget), "drawingarea1"));
  }

  /*if (opt_quit && end_of_rescal) {
    LogPrintf("quit\n");
    exit(-1);
  }*/

  return FALSE;
}


void on_window1_destroy (GtkObject *object, gpointer user_data) {
//#ifdef __linux__
//fclose(proc_net_dev);
//#endif
  if (gc) gdk_gc_unref (gc);
  gtk_main_quit();
#ifdef PARALLEL
  if (mode_par) synchro_quit();
#endif
  view_quit();
  //rescal_quit();
}

void timer_init(GtkWidget *widget) {
  GtkWidget  *drawingarea = lookup_widget(GTK_WIDGET (widget), "drawingarea1");
  if (!frame_delay){
    float flash_delay = get_flash_delay(drawingarea) * 1000.0;
    //LogPrintf("frame_delay = %f\n", frame_delay);
    LogPrintf("flash_delay_avg = %f\n", flash_delay);
    //if (flash_delay < frame_delay) frame_delay -= flash_delay;
    //else frame_delay = flash_delay + frame_delay;
    //if (frame_delay < flash_delay / 2.0) frame_delay = flash_delay / 2.0;
    frame_delay = flash_delay * 5;
    if (frame_delay < 10) frame_delay = 10;
  }
  LogPrintf("frame_delay = %f\n", frame_delay);
  timer = gtk_timeout_add(frame_delay, gcallback_flash, drawingarea);
}

