/* ReSCAL - GTK interfaces
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
//#include "support.h"

extern gint area_w, area_h;

GtkWidget*
create_window1 (void)
{
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *hboxbut, *hboxsim;
  GtkWidget *button_pause, *button_quit;
  GtkWidget *drawingarea1;
  GtkWidget *label2;

#ifdef GTK2
  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
#else
  window1 = gtk_window_new (GTK_WINDOW_DIALOG);
#endif
  gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
  gtk_window_set_title (GTK_WINDOW (window1), "ReSCAL");

  // box for widgets
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window1), vbox1);

  // box for buttons
  hboxbut = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hboxbut);
  //gtk_container_add (GTK_CONTAINER (vbox1), hboxbut);
  gtk_box_pack_start(GTK_BOX(vbox1), hboxbut, FALSE, FALSE, 0);

  // pause button
  button_pause = gtk_button_new_with_label("Pause");
  g_signal_connect (G_OBJECT (button_pause), "clicked", G_CALLBACK (gcallback_pause), button_pause);
  gtk_box_pack_start(GTK_BOX(hboxbut), button_pause, TRUE, TRUE, 0);
  gtk_widget_show (button_pause);

  // quit button
  button_quit = gtk_button_new_with_label("Quit");
  g_signal_connect (G_OBJECT (button_quit), "clicked", G_CALLBACK (gcallback_stop), 0);
  gtk_box_pack_start(GTK_BOX(hboxbut), button_quit, TRUE, TRUE, 0);
  gtk_widget_show (button_quit);

  // box for simulation
  hboxsim = gtk_hbox_new (FALSE, 0);
  //gtk_object_set_data_full (GTK_OBJECT (vboxall), "vbox1", vboxsim,                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (vbox1), hboxsim);
  gtk_widget_show (hboxsim);

  // drawing area
  drawingarea1 = gtk_drawing_area_new ();
  gtk_widget_ref (drawingarea1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "drawingarea1", drawingarea1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (drawingarea1);
  gtk_box_pack_start (GTK_BOX (hboxsim), drawingarea1, TRUE, TRUE, 10);
  gtk_widget_set_usize (drawingarea1, area_w, area_h);

  // info label
  label2 = gtk_label_new ("");
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (hboxsim), label2, FALSE, FALSE, 0);
  gtk_widget_set_usize (label2, (area_w<175) ? 175 : area_w, -2);
  //gtk_widget_set_uposition(label2, 0, area_h+10);

/*
#ifdef GTK2
  PangoFontDescription *font_desc = pango_font_description_from_string("Fixed 11");
  pango_font_description_set_family (font_desc, "monospace");
  gtk_widget_modify_font(label2, font_desc);
#endif
*/

  gtk_signal_connect (GTK_OBJECT (window1), "destroy",
                      GTK_SIGNAL_FUNC (on_window1_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose_event",
                      GTK_SIGNAL_FUNC (on_drawingarea1_expose_event),
                      NULL);

  return window1;
}

