/* ReSCAL - Parsing of parameters
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "defs.h"
#include "macros_public.h"
#include "param.h"
#include "terre.h"
#include "collisions.h"

#define LG_MAX 1024 //longueur max d'une ligne dans le fichier des parametres

extern char *nom_fic_terre; //nom du fichier TERRE
extern int    H, L, P;           // les dimensions de la terre
extern double temps;          // temps reel simule

#if defined(MODELE_DUN)
extern float lambda_F, lambda_R, lambda_D, lambda_J, lambda_V, lambda_G, lambda_I, lambda_A;  // les  lambdas
#elif defined(MODELE_AVA)
extern float lambda_A;  // les  lambdas
#else
extern float lambda_I, lambda_D, lambda_L, lambda_C, lambda_R, lambda_E_v, lambda_E_h;  // les  lambdas
extern float gravite;  // intensite de la gravite
#endif

#if defined(MODELE_CMB) || defined(MODELE_RIV) || defined(MODELE_DUN)
extern float flux_eo, flux_ns;  // intensite flux est-ouest, flux nord-sud
#else
extern float flux;                        // intensite flux horizontal
#endif
extern float tr_v, tr_h, diff_h, debit;     //termes de transports verticaux/horizontaux, diffusion horizontale et debit
#ifdef SURRECTIONS
extern float delai_sur;              // delai entre deux surrections successives (0 = pas de surrection)
extern int sur_mode;                // type de surrection (uniforme ou locale)
#endif
#if defined(AVALANCHES) || defined(TRANSITION_AVA) || defined(PROPAGATION_AVA)
extern int ava_h_lim;  // hauteur limite avant avalanche
extern int ava_nb_cel_max;  // nb de cellules qui tombent simultanement
extern float ava_delai; //delai entre avalanches
extern float ava_angle; //angle d'avalanches (degres)
extern float ava_angle2; //angle d'avalanches (degres)
#endif
#ifdef COLLISIONS
extern char *nom_fic_mvt; //nom du fichier VELOC
extern int init_mvt_nb_cycles;
#ifdef CGV
extern float grdvc;   //constante critique pour le controle de l'erosion en fonction du gradient de vitesse
extern float grdvc_max; //seuil max pour le controle de l'erosion en fonction du gradient de vitesse
extern float grdvc_min; //seuil min pour le controle de l'erosion en fonction du gradient de vitesse
#endif
#ifdef MVT_GFOR
extern float intensite_gfor; //instensite du forcage global
#endif
#endif
#ifdef CELL_COLOR
extern float color_coef;         // coefficient associe a la callback check_color
#endif
#ifdef SURRECTIONS
extern int sur_mode;
extern float sur_delay;
char *sur_mode_str=NULL;
#endif
#ifdef ROTATIONS
extern int rot_mode;
extern float rot_angle;
extern float rot_delay;
#ifdef COLLISIONS
extern int rot_mvt_nb_cycles;
#endif
char *rot_mode_str=NULL;
#endif

int nb_params=0;    //nombre de parametres
Parametre *liste_params=NULL;  //liste des parametres

//DECLARATION D'UN PARAMETRE
void parametre(char *par_nom, char *par_usage, void *par_adr, unsigned char par_type);
// par_nom : nom du parametre
// par_usage : definition du parametre
// par_adr : adresse du parametre (en memoire)
// par_type : type du parametre : PARAM_STRING | PARAM_INT | PARAM_FLOAT | PARAM_DOUBLE


void init_liste_params()
{
  //allocation memoire
  //liste_params = (Parametre*) malloc(NB_PARAM_MAX*sizeof(Parametre));
  AllocMemory(liste_params,Parametre,NB_PARAM_MAX);
  ResetMemory(liste_params,Parametre,NB_PARAM_MAX);

  //dimensions de la Terre
  parametre("H", "height", &H, PARAM_INT);
  parametre("L", "length", &L, PARAM_INT);
  parametre("D", "depth", &P, PARAM_INT);

  //fichier TERRE
  parametre("Bin_file", "initial cellular space (binary file)", &nom_fic_terre, PARAM_STRING);
  //parametre("Csp_file", "initial cellular space (csp file)", &nom_fic_terre, PARAM_STRING);

#ifdef COLLISIONS
  //fichier MVT
  parametre("Mvt_file", "initial lattice gas (mvt file) - optional", &nom_fic_mvt, PARAM_STRING);
  parametre("Init_ncycl", "initial number of flow cycles without transitions - optional", &init_mvt_nb_cycles, PARAM_INT);
#endif

  //temps
  parametre("Time", "initial physical time", &temps, PARAM_DOUBLE);

#ifdef MODELE_DUN
  //lambdas
  parametre("Lambda_F", "erosion rate", &lambda_F, PARAM_FLOAT);
  parametre("Lambda_R", "deposition rate", &lambda_R, PARAM_FLOAT);
  parametre("Lambda_V", "diffusion rate", &lambda_V, PARAM_FLOAT);
  parametre("Lambda_G", "gravitational rate", &lambda_G, PARAM_FLOAT);
  parametre("Lambda_I", "injection rate - optional", &lambda_I, PARAM_FLOAT);
  parametre("Lambda_A", "avalanches rate - optional", &lambda_A, PARAM_FLOAT);
  //parametre("Lambda_D", "non-utilisé", &lambda_D, PARAM_FLOAT);
  //parametre("Lambda_J", "non-utilisé", &lambda_J, PARAM_FLOAT);
#ifdef AVALANCHES
  parametre("Ava_h_lim", "height limit in avalanches (5 by default)", &ava_h_lim, PARAM_INT);
  parametre("Ava_nb_cel", "number of cells falling together (2 by default)", &ava_nb_cel_max, PARAM_INT);
  parametre("Ava_delay", "delay between avalanches (1.0 by default)", &ava_delai, PARAM_FLOAT);
#ifdef AVA_NORM
  parametre("Ava_angle", "slope angle in avalanches (in degrees)", &ava_angle, PARAM_FLOAT);
#endif
#endif
#ifdef TRANSITION_AVA
  parametre("Ava_h_lim", "height limit in avalanches (5 by default)", &ava_h_lim, PARAM_INT);
#ifdef AVA_NORM
  parametre("Ava_angle", "slope angle in avalanches (degrees)", &ava_angle, PARAM_FLOAT);
#ifdef CELL_COLOR
  parametre("Ava_angle2", "slope angle in avalanches of colored grains (degrees)", &ava_angle2, PARAM_FLOAT);
#endif
#endif
#endif
#ifdef PROPAGATION_AVA
  parametre("Ava_delay", "delay between avalanches (10.0 by default)", &ava_delai, PARAM_FLOAT);
#endif
#ifdef CGV
//  parametre("Grdvc", "critical shear stress for relationship between erosion rate and shear stress - optional", &grdvc, PARAM_FLOAT);
  parametre("Grdvc_max", "max shear stress value in the linear relationship between erosion rate and shear stress - optional", &grdvc_max, PARAM_FLOAT);
  parametre("Grdvc_min", "min shear stress value in the linear relationship between erosion rate and shear stress - optional", &grdvc_min, PARAM_FLOAT);
#endif
#ifdef MVT_GFOR
  parametre("Mvt_gfor", "global forcing of the flow - optional", &intensite_gfor, PARAM_FLOAT);
#endif
#ifdef CELL_COLOR
  parametre("Color_coef", "control coefficient of colored grains - optional", &color_coef, PARAM_FLOAT);
#endif
#endif

#ifdef MODELE_CHE
  //lambdas
  parametre("Lambda_I", "coefficient d'infiltration", &lambda_I, PARAM_FLOAT);
  parametre("Lambda_D", "coefficient de diffusion boue/eau", &lambda_D, PARAM_FLOAT);
  parametre("Lambda_L", "coefficient de dissolution", &lambda_L, PARAM_FLOAT);
  parametre("Lambda_C", "coefficient de cristallisation", &lambda_C, PARAM_FLOAT);
  parametre("Gravite", "coefficient de gravite boue/eau", &gravite, PARAM_FLOAT);
  parametre("Flux", "transport horizontal boue/eau", &flux, PARAM_FLOAT);
  parametre("Tr_v", "transport vertical eau/air", &tr_v, PARAM_FLOAT);
  parametre("Tr_h", "transport horizontal eau/air", &tr_h, PARAM_FLOAT);
  parametre("Diff_h", "diffusion horizontale eau/air", &diff_h, PARAM_FLOAT);
  parametre("Debit", "debit de l'eau en entree", &debit, PARAM_FLOAT);
#endif

#ifdef MODELE_RIV
  //lambdas
  parametre("Lambda_I", "coefficient d'infiltration", &lambda_I, PARAM_FLOAT);
  parametre("Lambda_D", "coefficient de diffusion boue/eau", &lambda_D, PARAM_FLOAT);
  parametre("Lambda_L", "coefficient de dissolution", &lambda_L, PARAM_FLOAT);
  parametre("Lambda_E_v", "coefficient d'erosion verticale par transport", &lambda_E_v, PARAM_FLOAT);
  parametre("Lambda_E_h", "coefficient d'erosion horizontale par transport", &lambda_E_h, PARAM_FLOAT);
  parametre("Lambda_C", "coefficient de cristallisation", &lambda_C, PARAM_FLOAT);
  parametre("Gravite", "coefficient de gravite boue/eau", &gravite, PARAM_FLOAT);
  parametre("Flux_eo", "intensite du flux est-ouest", &flux_eo, PARAM_FLOAT);
  parametre("Flux_ns", "intensite du flux nord-sud", &flux_ns, PARAM_FLOAT);
  parametre("Debit", "debit a la source", &debit, PARAM_FLOAT);
#ifdef AVALANCHES
  parametre("Ava_h_lim", "hauteur limite avant avalanche (5 par defaut)", &ava_h_lim, PARAM_INT);
  parametre("Ava_nb_cel", "nombre de cellules qui tombent simultanement (2 par defaut)", &ava_nb_cel_max, PARAM_INT);
  parametre("Ava_delai", "delai entre avalanches (1.0 par defaut)", &ava_delai, PARAM_FLOAT);
#endif
#endif

#if defined(MODELE_CMB) || defined(MODELE_CRI)
  //lambdas
  parametre("Lambda_I", "coefficient d'infiltration", &lambda_I, PARAM_FLOAT);
  parametre("Lambda_D", "coefficient de diffusion", &lambda_D, PARAM_FLOAT);
  parametre("Lambda_L", "coefficient de dissolution", &lambda_L, PARAM_FLOAT);
  parametre("Lambda_C", "coefficient de cristallisation", &lambda_C, PARAM_FLOAT);
  parametre("Gravite", "coefficient de gravite", &gravite, PARAM_FLOAT);
  parametre("Flux_eo", "intensite du flux est-ouest", &flux_eo, PARAM_FLOAT);
  parametre("Flux_ns", "intensite du flux nord-sud", &flux_ns, PARAM_FLOAT);
#endif

#if defined(MODELE_AVA)
  //lambdas
  parametre("Ava_h_lim", "hauteur limite avant avalanche", &ava_h_lim, PARAM_INT);
#ifdef AVALANCHES
  parametre("Ava_nb_cel", "nombre de cellules qui tombent simultanement", &ava_nb_cel_max, PARAM_INT);
  parametre("Ava_delai", "delai entre avalanches (10.0 par defaut)", &ava_delai, PARAM_FLOAT);
#ifdef AVA_NORM
  parametre("Ava_angle", "angle d'avalanches (degres)", &ava_angle, PARAM_FLOAT);
#endif
#endif //AVALANCHES
#ifdef TRANSITION_AVA
  parametre("Lambda_A", "avalanches (en option)", &lambda_A, PARAM_FLOAT);
#ifdef AVA_NORM
  parametre("Ava_angle", "angle d'avalanches (degres)", &ava_angle, PARAM_FLOAT);
#ifdef CELL_COLOR
  parametre("Ava_angle2", "angle d'avalanches des grains blancs (degres)", &ava_angle2, PARAM_FLOAT);
#endif
#endif
#endif //TRANSITION_AVA
#ifdef PROPAGATION_AVA
  parametre("Ava_delai", "delai entre avalanches (10.0 par defaut)", &ava_delai, PARAM_FLOAT);
#endif
#endif //MODELE_AVA

#ifdef MODELE_ICB
  //lambdas
  //parametre("lambda_P", "coefficient de compaction", &lambda_P, PARAM_FLOAT);
  parametre("Lambda_R", "coefficient de refroidissement", &lambda_R, PARAM_FLOAT);
  parametre("Lambda_D", "coefficient de diffusion", &lambda_D, PARAM_FLOAT);
  parametre("Lambda_L", "coefficient de dissolution", &lambda_L, PARAM_FLOAT);
  parametre("Lambda_C", "coefficient de cristallisation", &lambda_C, PARAM_FLOAT);
  parametre("Gravite", "ntensite de la gravite", &gravite, PARAM_FLOAT);
  parametre("Flux", "intensite du flux horizontal", &flux, PARAM_FLOAT);
#endif

#ifdef MODELE_DIF
  //lambdas
  parametre("Lambda_D", "coefficient de diffusion", &lambda_D, PARAM_FLOAT);
  parametre("Debit", "debit a la source", &debit, PARAM_FLOAT);
#endif

#ifdef SURRECTIONS
  parametre("Sur_mode", "surrection mode (UNIFORM|CONE|TECTO)", &sur_mode_str, PARAM_STRING);
  parametre("Sur_delay", "surrection delay", &delai_sur, PARAM_FLOAT);
#endif
#ifdef ROTATIONS
  parametre("Rot_mode", "rotation mode (DISK|CYCLE) - optional", &rot_mode_str, PARAM_STRING);
  parametre("Rot_angle", "rotation angle - optional", &rot_angle, PARAM_FLOAT);
  parametre("Rot_delay", "rotation delay - optional", &rot_delay, PARAM_FLOAT);
#ifdef COLLISIONS
  parametre("Rot_ncycl", "number of flow cycles without transitions after rotation - optional", &rot_mvt_nb_cycles, PARAM_INT);
#endif
#endif
}


void param_usage()
{
  int i, l;

  printf("FORMAT OF PARAMETERS_FILE\n");
  printf("  <parameter> = <value>\n");
  printf("  List of parameters:\n");

  init_liste_params();

  for(i=0; i<nb_params; i++){
    l = strlen(liste_params[i].nom);
    printf("    %s%s %s\n", liste_params[i].nom, (l<=2)?"\t\t":"\t", liste_params[i].usage);
  }

}

void bad_args()
{
  ErrPrintf("erreur lecture arguments\n");

  exit(-4);
}

void bad_params()
{
  ErrPrintf("erreur lecture parametres\n");

  exit(-4);
}


int read_int(char* s)
{
  if (!isdigit(s[0]) && ((s[0]!='-') || !isdigit(s[1]))) bad_args();
  return atoi(s);
}

float read_float(char* s)
{
  //LogPrintf("read_float : %s\n",s);
  if (!isdigit(s[0]) && ((s[0]!='-') || !isdigit(s[1]))) bad_args();
  return atof(s);
}

void parametre(char *par_nom, char *par_usage, void *par_adr, unsigned char par_type)
{
  liste_params[nb_params].nom = par_nom;
  liste_params[nb_params].usage = par_usage;
  liste_params[nb_params].param = par_adr;
  liste_params[nb_params].type = par_type;
  nb_params++;
}

void init_val_param(char *par_nom, char *par_val)
{
  int i=0;

  while ((i<nb_params) && (strcmp(liste_params[i].nom, par_nom))) i++;

  if (i<nb_params){
    switch (liste_params[i].type) {
      case PARAM_STRING:
        *(char**)liste_params[i].param = (char*) malloc(strlen(par_val)+1);
        strcpy(*(char**)liste_params[i].param, par_val);
        break;
      case PARAM_INT:
        *(int*)liste_params[i].param = read_int(par_val);
        break;
      case PARAM_FLOAT:
        *(float*)liste_params[i].param = read_float(par_val);
        break;
      case PARAM_DOUBLE:
        *(double*)liste_params[i].param = read_float(par_val);
        break;
    }
    liste_params[i].init = 1;
    LogPrintf("%s = %s\n", par_nom, par_val);
  }
  else{
    ErrPrintf("ERREUR : parametre inconnu - %s\n", par_nom);

    exit(-4);
  }
}

void lecture_ligne(FILE *fp, char *str)
{
  char *ptr, *ptrmax, c;

  ptr = str;
  ptrmax = str+LG_MAX;

  // on lit une ligne complete, sans les espaces
  while (!feof(fp) && (ptr<ptrmax)){
    c = fgetc(fp);
    if ((c == '\n') || (c == EOF)){
      break;
    }
    else if (c != ' '){
      *ptr++ = c;
    }
  }

  if (ptr>=ptrmax) bad_params();

  *ptr=0;
}

void param_parse()
{
#ifdef SURRECTIONS
  if (sur_mode_str){
    if (!strcmp(sur_mode_str, "UNIFORM")){
      sur_mode = SUR_MODE_UNIFORM;
    }
    else if (!strcmp(sur_mode_str, "CONE")){
      sur_mode = SUR_MODE_CONE;
    }
    else if (!strcmp(sur_mode_str, "TECTO")){
      sur_mode = SUR_MODE_TECTO;
    }
  }
#endif
#ifdef ROTATIONS
  if (rot_mode_str){
    if (!strcmp(rot_mode_str, "DISK")){
      rot_mode = ROT_MODE_DISK;
    }
    else if (!strcmp(rot_mode_str, "CYCLE")){
      rot_mode = ROT_MODE_CYCL;
    }
  }
#endif
}

void lecture_parametres(char *param_file)
{
  FILE *fp;
  char *par_nom;
  char *par_val;
  char str[LG_MAX];
  char *ptr;
  int n=0;

  //initialisation de la liste des parametres du modele
  init_liste_params();

  LogPrintf("modele %s\n",MOD_NAME);
  LogPrintf("lecture fichier parametres : %s\n", param_file);

  fp = fopen(param_file,"r");
  if (!fp){
    ErrPrintf("erreur ouverture fichier parametres : %s\n", param_file);
    exit(-4);
  }

  while (!feof(fp)){
    *str = 0;
    lecture_ligne(fp, str);
    //LogPrintf("str=%s\n", str);
    if (*str == '#'){  //c'est un commentaire, on ne lit pas cette ligne
      //LogPrintf("commentaire ...\n");
    }
    else if (*str == 0){ //c'est une ligne vide
      //LogPrintf("ligne vide ...\n");
    }
    else {
      ptr = strchr(str,'=');
      if (ptr){
        // on decoupe la chaine str au niveau du caractere '='
        *ptr = 0;
        par_nom = str;
        par_val = ptr+1;
        init_val_param(par_nom, par_val);
        n++;
      }
      else
        bad_params();
    }
  }

  fclose(fp);

  LogPrintf("lecture_parametres : %d parametres lus\n", n);

  if (n<nb_params){
    int i=0;
    for (i=0; i<nb_params; i++)
      if (!liste_params[i].init) LogPrintf("ATTENTION : parametre %s non-trouve\n", liste_params[i].nom);
  }

  param_parse();
}

